#define GAMMA 2.0

// Textures
uniform lowp sampler2D u_texture; // Diffuse Texture
uniform lowp sampler2D u_texture1; // Normalmap Texture

// Varying Inputs
varying mediump vec3 v_normal;
varying mediump vec3 v_tangent;
varying mediump vec3 v_binormal;

varying lowp vec2 v_texCoord;
varying mediump vec3 v_lightDir;
varying mediump vec3 v_halfDir;

varying mediump vec3 v_worldPos;
varying mediump vec3 v_camPos;
varying mediump vec3 v_viewDir;

// Engine Uniforms
uniform lowp vec4 u_directionalLightColor;
uniform lowp vec4 u_materialAmbient;
uniform lowp vec4 u_materialDiffuse;
uniform lowp vec4 u_materialSpecular;
uniform lowp float u_materialShininess;

// Custom Uniforms
// x = Tile Factor
// y = Normalmap Strength
// z = Specular Strength
// w = Diffuse term
uniform mediump vec4 Settings;

void main() {
	// Tile texcoords (does not work with atlases!)
	mediump vec2 tiled = v_texCoord * Settings.x;

	// Sample diffuse texture
	lowp vec4 diffuseMap = texture2D(u_texture, tiled);
	// Sample normal texture
	lowp vec3 normalMap = texture2D(u_texture1, tiled).rgb;
	normalMap = mix(vec3(0.5, 0.5, 1.0), normalMap, clamp(Settings.y, 0.0, 1.0));
	normalMap = normalMap * 2.0 - 1.0;
	
	mediump vec3 Nv = normalize(v_normal);
	mediump vec3 Tv = normalize(v_tangent);
	mediump vec3 Bv = normalize(v_binormal);

	mediump mat3 TBN = mat3(Tv, Bv, Nv);
	mediump vec3 N = normalize(TBN * normalMap);
	
	// Colors
	lowp vec4 matAmbient = u_materialAmbient;
	lowp vec4 matDiffuse = u_materialDiffuse;
	lowp vec4 matSpecular = u_materialSpecular;
	lowp vec4 lightColor = u_directionalLightColor;
	
	// Gamma correct colors (Use 2.0 to use sqrt instead of pow)
	diffuseMap.rgb = pow(diffuseMap.rgb, vec3(GAMMA, GAMMA, GAMMA));

	// Normal (normalmap)
	mediump vec3 L = normalize(v_lightDir);
	mediump vec3 V = normalize(v_camPos - v_worldPos);
	mediump vec3 H = normalize(L + V);

	// Calcuate lighting factors
	mediump float NdotL = max(dot(N, L), 0.0);
	mediump float NdotH = max(dot(N, H), 0.0);
	
	NdotL = (NdotL + Settings.w) / (1.0 + Settings.w);
	NdotL = clamp(NdotL, 0.0, 1.0);

	// Diffuse
	lowp vec3 diffuseColor = lightColor.rgb *
				 matDiffuse.rgb *
				 diffuseMap.rgb *
				 NdotL;

	// Specular
	mediump float gloss = max(u_materialShininess * 8.0, 1.0);
	mediump float spec = max(pow(NdotH, gloss) * Settings.z, 0.0);
	lowp vec3 specularColor = matSpecular.rgb * spec;

	// Composed
	lowp vec3 finalColor = matAmbient.rgb;
	finalColor = finalColor + diffuseColor;
	finalColor = finalColor + specularColor;
	
	finalColor = sqrt(finalColor);

	gl_FragColor = vec4(finalColor, diffuseMap.a);
}
